@@ -3,8 +3,8 @@
 ;; Copyright (C) 2019-2020 Mihai Olteanu
 
 ;; Author: Mihai Olteanu <mihai_olteanu@fastmail.fm>
-;; Package-Version: 1.2
-;; Package-Revision: 96568f07324b
+;; Package-Version: 20260130.1102
+;; Package-Revision: d095474f2264
 ;; Package-Requires: ((emacs "26.1") (request "0.3.0") (anaphora "1.0.4") (memoize "1.1") (elquery "0.1.0") (s "1.12.0"))
 ;; Keywords: multimedia, api
 ;; URL: https://github.com/mihaiolteanu/lastfm.el/
@@ -142,12 +142,6 @@ README_api.md if `lastfm-enable-doc-generation' is t.")
     (lastfm--set-config-parameters))
   lastfm--sk)
 
-(defun lastfm--first-value (resp)
-  "Extract the very first value from the RESP.
-RESP usually contains a single Last.fm response element, like a
-token or sk.  Return the value of that."
-  (cdaar resp))
-
 (defun lastfm-generate-session-key ()
   "Generate a session key and save it in the .lastfmrc file.
 The user needs to grant Last.fm access to his/her application.
@@ -155,7 +149,7 @@ Both `lastfm-auth-get-token' and `lastfm-auth-get-session' methods used
 here are generated with 'lastfm--defmethod'."
   ;; Reload after the user updated the empty .lastfmrc file.
   (lastfm--set-config-parameters)
-  (let ((token (lastfm--first-value (lastfm-auth-get-token))))
+  (let ((token (car (lastfm-auth-get-token))))
     ;; Ask the user to allow access.
     (browse-url (concat "http://www.last.fm/api/auth/?api_key="
                         (lastfm--api-key)
@@ -163,7 +157,7 @@ here are generated with 'lastfm--defmethod'."
     (when (yes-or-no-p "Did you grant the application persmission
 to access your Last.fm account? ")
       ;; If permission granted, get the sk and update the config file.
-      (let* ((sk (lastfm--first-value (lastfm-auth-get-session token)))
+      (let* ((sk (car (lastfm-auth-get-session token)))
              (config (lastfm--read-config-file))
              (config-with-sk (append config (list :SK sk))))
         (with-temp-file (lastfm--config-file)
@@ -297,7 +291,7 @@ extract and to build the request response."
 
 (lastfm--defmethod artist.getTopTracks (artist (limit 10) (page 1))
   "Get the top tracks by an artist, ordered by popularity."
-  :no ("track artist name" "track > name" "playcount" "listeners"))
+  :no ("artist name" "name" "playcount" "listeners"))
 
 (lastfm--defmethod artist.removeTag (artist tag)
   "Remove a user's tag from an artist."
@@ -521,11 +515,30 @@ VALUES."
   "Send and return the Last.fm request for METHOD.
 AUTH, PARAMS and VALUES are only passed allong to
 'lastfm--build-params'.  See the documentation for that method."
+  (let ((data (lastfm--build-params method auth params values)))    
+    (if (eql auth :yes)
+        (lastfm--post-request data)
+      (lastfm--get-request data))))
+
+(defun lastfm--post-request (data)
+  "POST lastfm request for DATA."
   (let (resp)
     (request lastfm--url
-             :params (lastfm--build-params method auth params values)
-             :parser 'buffer-string
              :type   "POST"
+             :data   data
+             :parser 'buffer-string             
+             :sync   t
+             :complete (cl-function
+                        (lambda (&key data &allow-other-keys)
+                          (setq resp data))))
+    resp))
+
+(defun lastfm--get-request (data)
+  "GET lastfm request for DATA."  
+  (let (resp)
+    (request lastfm--url
+             :params data
+             :parser 'buffer-string
              :sync   t
              :complete (cl-function
                         (lambda (&key data &allow-other-keys)
@@ -538,8 +551,8 @@ Each string from the QUERY-STRINGS list of strings contains one
 CSS selector to extract the given HTML elements from the Last.fm
 RESPONSE string."
   (let ((raw-response (elquery-read-string response)))
-    (awhen (elquery-text
-            (cl-first (elquery-$ "error" raw-response)))
+    (awhen (s-present-p (elquery-text
+                         (cl-first (elquery-$ "error" raw-response))))
       (error it))
     (let ((parsed-response
            ;; Parse the last.fm raw response with each of the query-strings
